package org.photon.core.login.db;

import com.google.common.primitives.Bytes;
import com.google.inject.Inject;

import java.io.UnsupportedEncodingException;
import java.security.MessageDigest;

/**
 * @author Blackrush
 */
public final class DefaultLoginCryptoService implements LoginCryptoService {
    public static final String CHARSET = "UTF-8";
    public static final String EXTENDED_ALPHABET = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-_";

    private final MessageDigest digest;

    @Inject
    public DefaultLoginCryptoService(MessageDigest digest) {
        this.digest = digest;
    }

    /**
     * encrypt a password using a salt
     * @param clearPassword the password to encrypt
     * @param salt the salt to use for password's encryption
     * @return the encrypted password using the salt
     */
    @Override
    public String cipher(String clearPassword, String salt) {
        try {
            byte[] hashPassword = digest.digest(clearPassword.getBytes(CHARSET));
            byte[] hash = digest.digest(Bytes.concat(hashPassword, salt.getBytes(CHARSET)));
            return new String(hash, CHARSET);
        } catch (UnsupportedEncodingException e) {
            throw new Error(e);
        }
    }

    /**
     * Dofus 1.x password decryption algorithm (all credits to Ancestra)
     * @param hashpass the encrypted password to decrypt
     * @param key the key used to encrypt the password
     * @return the decrypted password
     */
    @Override
    public String decrypt(String hashpass, String key) {
        StringBuilder decrypted = new StringBuilder(hashpass.length() / 2);
        int length = EXTENDED_ALPHABET.length();

        int PKey, ANB, ANB2, somme1, somme2, APass, AKey;

        for (int i = 0; i < hashpass.length(); i+=2)
        {
            PKey = key.charAt(i/2);
            ANB = EXTENDED_ALPHABET.indexOf(hashpass.charAt(i));
            ANB2 = EXTENDED_ALPHABET.indexOf(hashpass.charAt(i+1));

            somme1 = ANB + length;
            somme2 = ANB2 + length;

            APass = somme1 - PKey;
            if (APass < 0) APass += 64;
            APass = APass << 4;

            AKey = somme2 - PKey;
            if (AKey < 0) AKey += 64;

            char PPass = (char) (APass + AKey);

            decrypted.append(PPass);
        }

        return decrypted.toString();
    }
}
