package org.photon.core.login.db;

import com.google.inject.Inject;
import fi.evident.dalesbred.Database;
import fi.evident.dalesbred.NonUniqueResultException;

/**
 * @author Blackrush
 */
public final class DefaultUserRepository implements UserRepository {
    @Inject
    private Database db;

    @Inject
    private LoginCryptoService crypto;

    @Override
    public User createEmpty() {
        return new User();
    }

    @Override
    public User findById(Long id) {
        try {
            User user = db.findUnique(User.class,
                    "select id, username, password_hash, salt, banned, nickname, secret_question, secret_answer " +
                    "from users where id=?", id);
            user.setPersisted(true);
            return user;
        } catch (NonUniqueResultException e) {
            return null;
        }
    }

    @Override
    public User findByUsername(String username) {
        try {
            User user = db.findUnique(User.class,
                    "select id, username, password_hash, salt, banned, nickname, secret_question, secret_answer " +
                    "from users where username=?", username);
            user.setPersisted(true);
            return user;
        } catch (NonUniqueResultException e) {
            return null;
        }
    }

    @Override
    public User authenticate(String username, String hashpass, String key) {
        User user = findByUsername(username);

        if (user == null) {
            throw new AuthenticationException.InvalidCredentials();
        }

        String passwordHash = crypto.cipher(crypto.decrypt(hashpass, key), user.getSalt());

        if (!user.getPasswordHash().equals(passwordHash)) {
            throw new AuthenticationException.InvalidCredentials();
        }

        if (user.isBanned()) {
            throw new AuthenticationException.Banned();
        }

        return user;
    }

    @Override
    public void persist(User user) {
        if (user.isPersisted()) {
            db.update("update users set " +
                      "username=:username, password_hash=:passwordHash, salt=:salt, banned=:banned, nickname=:nickname, " +
                      "secret_question=:secret_question, secret_answer=:secret_answer " +
                      "where id=:id", user);
        } else {
            db.update("insert into users(username, password_hash, salt, banned, nickname, secret_question, secret_answer) " +
                      "values(:username, :passwordHash, :salt, :banned, :nickname, :secret_question, :secret_answer)", user);

            user.setPersisted(true);
        }
    }

    @Override
    public void remove(User user) {
        if (!user.isPersisted()) {
            return;
        }
        db.update("delete from users where id=:id", user);
    }
}
