package org.photon.core.login.db;

import com.google.inject.AbstractModule;
import com.google.inject.Provides;
import com.google.inject.Scopes;
import com.google.inject.Singleton;
import com.typesafe.config.Config;
import fi.evident.dalesbred.connection.DriverManagerDataSourceProvider;
import fi.evident.dalesbred.support.guice.DataSourceDatabaseModule;

import javax.sql.DataSource;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

/**
 * @author Blackrush
 */
public class PhotonLoginDatabaseModule extends AbstractModule {
    @Override
    protected void configure() {
        bind(UserRepository.class).to(DefaultUserRepository.class).in(Scopes.SINGLETON);

        install(new DataSourceDatabaseModule());
    }

    @Singleton
    @Provides
    DataSource provideDataSource(Config config) {
        try {
            Class.forName(config.getString("photon.login.db.driver"));
            return DriverManagerDataSourceProvider.createDataSource(
                    config.getString("photon.login.db.url"),
                    config.getString("photon.login.db.user"),
                    config.getString("photon.login.db.password")
            );
        } catch (ClassNotFoundException e) {
            addError(e);
            return null;
        }
    }

    @Singleton
    @Provides
    LoginCryptoService provideCrypto(Config config) {
        try {
            return new DefaultLoginCryptoService(MessageDigest.getInstance(config.getString("photon.login.db.digest_algorithm")));
        } catch (NoSuchAlgorithmException e) {
            throw new Error(e);
        }
    }
}
