package org.photon.core.login.db;

/**
 * @author Blackrush
 */
public interface UserRepository {
    /**
     * @return a non-null zero initialized user
     */
    User createEmpty();

    /**
     * find an user by its id
     * @param id user's id
     * @return found user or null
     */
    User findById(Long id);

    /**
     * find an user by its username
     * @param username user's username
     * @return found user or null
     */
    User findByUsername(String username);

    /**
     * find an user by its username and verify the password
     * @param username user's username
     * @param hashpass user's hash password
     * @param key user's key
     * @return a non-null user
     * @throws AuthenticationException
     */
    User authenticate(String username, String hashpass, String key);

    /**
     * insert or update user's values
     * @param user user to be persisted
     */
    void persist(User user);

    /**
     * delete user
     * @param user user to be removed
     */
    void remove(User user);
}
