package org.photon.core.login.net;

import com.google.inject.Inject;
import org.apache.mina.core.session.IoSession;
import org.photon.core.login.db.User;
import org.photon.core.protocol.client.ClientState;
import org.photon.core.protocol.client.PhotonMinaCodecFactory;

/**
 * @author Blackrush
 */
public final class DefaultLoginClient implements LoginClient {
    private final IoSession session;

    private String key;
    private User user;

    public DefaultLoginClient(IoSession session) {
        this.session = session;
    }

    @Override
    public String getKey() {
        return key;
    }

    @Inject
    public void setKey(LoginKeyProvider keyProvider) {
        this.key = keyProvider.get();
    }

    @Override
    public ClientState getState() {
        return (ClientState) session.getAttribute(PhotonMinaCodecFactory.CLIENT_STATE_ATTR);
    }

    @Override
    public DefaultLoginClient setState(ClientState state) {
        session.setAttribute(PhotonMinaCodecFactory.CLIENT_STATE_ATTR, state);
        return this;
    }

    @Override
    public User getUser() {
        return user;
    }

    @Override
    public DefaultLoginClient setUser(User user) {
        this.user = user;
        return this;
    }

    @Override
    public DefaultLoginClient write(Object msg) {
        session.write(msg);
        return this;
    }

    @Override
    public void kick() {
        session.close(true);
    }
}
