package org.photon.core.login.net;

import com.google.inject.PrivateModule;
import com.google.inject.Provides;
import com.google.inject.Scopes;
import com.google.inject.Singleton;
import com.google.inject.multibindings.Multibinder;
import org.apache.mina.filter.codec.ProtocolCodecFactory;
import org.photon.core.login.net.event.LoginNetworkEventDispatcherStrategy;
import org.photon.core.login.net.game.DefaultLoginGameManager;
import org.photon.core.login.net.game.LoginGameManager;
import org.photon.core.login.net.handler.DefaultLoginNetworkHandler;
import org.photon.core.protocol.client.Messages;
import org.photon.core.protocol.client.PhotonMinaCodecFactory;
import org.photon.shared.event.EventDispatcherStrategy;

/**
 * @author Blackrush
 */
public class PhotonLoginNetworkModule extends PrivateModule {
    @Override
    protected void configure() {
        bind(LoginNetworkService.class).to(DefaultLoginNetworkService.class).in(Scopes.SINGLETON);
        bind(EventDispatcherStrategy.class).toInstance(new LoginNetworkEventDispatcherStrategy());
        bind(LoginKeyProvider.class).to(SecureLoginKeyProvider.class).in(Scopes.SINGLETON);
        bind(LoginGameManager.class).to(DefaultLoginGameManager.class).in(Scopes.SINGLETON);

        Multibinder b = Multibinder.newSetBinder(binder(), LoginNetworkHandler.class);
        b.addBinding().to(DefaultLoginNetworkHandler.class).in(Scopes.SINGLETON);

        expose(LoginNetworkService.class);
        expose(LoginGameManager.class);
    }

    @Singleton
    @Provides
    ProtocolCodecFactory provideProtocolCodecFactory() {
        return new PhotonMinaCodecFactory(Messages.fromClient());
    }
}
