package org.photon.core.login.net.event;

import org.photon.core.login.net.LoginClient;
import org.photon.core.protocol.client.Message;
import org.photon.shared.event.InvokerInterface;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import static com.google.common.base.Preconditions.checkArgument;
import static com.google.common.base.Throwables.propagate;

/**
 * @author Blackrush
 */
public class LoginHandleClientMessageInvoker implements InvokerInterface {
    private final Method method;
    private final Class<?> messageClass;

    public LoginHandleClientMessageInvoker(Method method) {
        checkArgument(method.getParameterTypes().length == 2,
                "%s must have 2 parameters : LoginClient and Message", method.toGenericString());
        checkArgument(LoginClient.class.isAssignableFrom(method.getParameterTypes()[0]),
                "%s must have in first parameter a LoginClient", method.toGenericString());
        checkArgument(Message.class.isAssignableFrom(method.getParameterTypes()[1]),
                "%s must have in second parameter a message", method.toGenericString());

        this.method = method;
        this.method.setAccessible(true);
        this.messageClass = method.getParameterTypes()[1];
    }

    @Override
    public Object invoke(Object o, Object subscriber) {
        checkArgument(o instanceof HandleClientMessageEvent, "event is not a HandleClientMessageEvent");

        HandleClientMessageEvent event = (HandleClientMessageEvent) o;

        if (!messageClass.isAssignableFrom(event.getMessage().getClass())) {
            return null;
        }

        try {
            return method.invoke(subscriber, event.getClient(), event.getMessage());
        } catch (IllegalAccessException e) {
            throw propagate(e); // should not happen
        } catch (InvocationTargetException e) {
            throw propagate(e.getTargetException());
        }
    }
}
