package org.photon.core.login.net.game;

import org.apache.mina.core.session.IoSession;
import org.photon.core.login.net.game.event.GameServiceStateUpdatedEvent;
import org.photon.core.protocol.client.enums.WorldState;
import org.photon.core.protocol.client.types.GameServerType;
import org.photon.core.protocol.server.UpdateStateMessage;

import static com.google.common.base.Preconditions.checkState;

/**
 * @author Blackrush
 */
public final class DefaultLoginGameService implements LoginGameService {
    private final int id;
    private final LoginGameManager manager;

    private IoSession session;
    private String laddr;
    private WorldState state = WorldState.OFFLINE;
    private int completion;

    public DefaultLoginGameService(int id, LoginGameManager manager) {
        this.id = id;
        this.manager = manager;
    }

    @Override
    public int getId() {
        return id;
    }

    private void dispatchStateUpdated() {
        manager.getDispatcher().dispatch(new GameServiceStateUpdatedEvent(manager, this));
    }

    public void sync(IoSession session, String laddr) {
        checkState(!isSync(), "service #%d is already synchronized", id);
        this.session = session;
        this.laddr = laddr;

        dispatchStateUpdated();
    }

    public void desync() {
        session = null;
        laddr = null;
        state = WorldState.OFFLINE;
        completion = 0;

        dispatchStateUpdated();
    }

    public void onMessage(Object o) {
        if (o instanceof UpdateStateMessage) {
            UpdateStateMessage message = (UpdateStateMessage) o;
            state = message.getState();
            completion = message.getCompletion();

            dispatchStateUpdated();
        }
    }

    @Override
    public boolean isSync() {
        return session != null;
    }

    @Override
    public String getLaddr() {
        return laddr;
    }

    @Override
    public WorldState getState() {
        return state;
    }

    @Override
    public boolean isJoinable() {
        return state == WorldState.ONLINE;
    }

    @Override
    public int getCompletion() {
        return completion;
    }

    @Override
    public GameServerType toGameServerType() {
        return new GameServerType(id, laddr, state, completion, isJoinable());
    }
}
