package org.photon.core.protocol.client;

import com.google.common.collect.Iterators;
import com.google.inject.Provider;

import java.util.Arrays;
import java.util.Collection;
import java.util.List;

/**
 * @author Blackrush
 */
public final class Serializables {
    private Serializables() {}

    public static final Serializable BLANK = new Serializable() {
        @Override public void serialize(StringBuilder builder) { }
        @Override public void deserialize(String input) { }
    };

    public static Serializable many(final String delimiter, final Collection<? extends Serializable> serializables) {
        if (serializables.size() <= 0) return BLANK;
        if (serializables.size() == 1) return Iterators.getOnlyElement(serializables.iterator());

        return new Serializable() {
            public void serialize(StringBuilder builder) {
                boolean first = true;
                for (Serializable serializable : serializables) {
                    if (first) first = false;
                    else builder.append(delimiter);
                    serializable.serialize(builder);
                }
            }

            public void deserialize(String input) { }
        };
    }

    public static Serializable many(String delimiter, Serializable... serializables) {
        return many(delimiter, Arrays.asList(serializables));
    }

    public static <T extends Serializable> Serializable many(final String delimiter, final Provider<T> provider, final List<T> result) {
        return new Serializable() {
            public void serialize(StringBuilder builder) { }

            public void deserialize(String input) {
                for (String data : input.split(delimiter)) {
                    T res = provider.get();
                    res.deserialize(data);
                    result.add(res);
                }
            }
        };
    }
}
