package org.photon.core.protocol.client.messages;

import com.google.common.collect.ImmutableList;
import com.google.common.collect.Lists;
import org.photon.core.protocol.client.AbstractMessage;
import org.photon.core.protocol.client.Message;
import org.photon.core.protocol.client.MessageFactory;
import org.photon.core.protocol.client.Serializables;
import org.photon.core.protocol.client.types.GameServerType;

import java.util.Collection;
import java.util.List;

/**
 * @author Blackrush
 */
public class GameServersMessage extends AbstractMessage {
    public static final String HEADER = "AH";

    public static final MessageFactory FACTORY = new MessageFactory() {
        public Message buildMessage() {
            return new GameServersMessage();
        }
    };

    private List<GameServerType> servers;

    public GameServersMessage() {
        super(HEADER);
    }

    public GameServersMessage(Collection<GameServerType> servers) {
        this();
        this.servers = ImmutableList.copyOf(servers);
    }

    public List<GameServerType> getServers() {
        return servers;
    }

    public void setServers(Collection<GameServerType> servers) {
        this.servers = ImmutableList.copyOf(servers);
    }

    @Override
    public void serialize(StringBuilder builder) {
        builder.append(HEADER);
        Serializables.many("\\|", servers).serialize(builder);
    }

    @Override
    public void deserialize(String input) {
        servers = Lists.newArrayList();
        Serializables.many("\\|", GameServerType.PROVIDER, servers).deserialize(input.substring(HEADER.length()));
    }
}
