package org.photon.runner;

import com.google.inject.Stage;
import com.typesafe.config.Config;
import com.typesafe.config.ConfigFactory;
import org.apache.commons.cli.*;
import org.photon.shared.event.EventDispatcherStrategies;
import org.photon.shared.kernel.BundleLoaders;
import org.photon.shared.kernel.BundleRegistries;
import org.photon.shared.kernel.BundleRegistry;

import java.io.File;

/**
 * @author Blackrush
 */
public class Main {
    private static final Options options = new Options()
            .addOption("c", "config", true, "set the config path")
            .addOption("b", "bundles", true, "set the bundles installation path")
            .addOption("l", "libs", true, "set the libs installation path")
            .addOption("s", "stage", true, "set the application stage")
            .addOption("h", "help", false, "print this message")
    ;

    private static final CommandLineParser parser = new GnuParser();

    public static void main(String[] args) throws ParseException {
        final CommandLine cmd = parser.parse(options, args);

        if (cmd.hasOption('h')) {
            new HelpFormatter().printHelp("run", options);
            return;
        }

        final Config config = ConfigFactory.parseFileAnySyntax(new File(cmd.getOptionValue("config", "./config")));

        final BundleRegistry registry = BundleRegistries.factory()
                .withConfig(config)
                .withDispatcherStrategy(EventDispatcherStrategies.reflect())
                .withLoader(BundleLoaders.file(
                        cmd.getOptionValue("bundles", "./bundles"),
                        cmd.getOptionValue("libs", "./lib")
                ))
                .withStage(Stage.valueOf(cmd.getOptionValue("stage", Stage.DEVELOPMENT.name())))
                .simple();

        registry.load();

        Runtime.getRuntime().addShutdownHook(new Thread() {
            public void run() {
                registry.shutDown();
            }
        });

        registry.startUp();
    }
}
