package org.photon.shared.collect;

import java.util.Iterator;
import java.util.Map;

/**
 * @author Blackrush
 */
public abstract class AbstractStringTree<V> implements StringTree<V> {
    protected abstract Map<Character, StringChild<V>> getChildren();
    protected abstract StringChild<V> buildChild(Character key, V value);

    @Override
    public boolean hasChildren() {
        return !getChildren().isEmpty();
    }

    @Override
    public StringChild<V> get(Character key) {
        return getChildren().get(key);
    }

    @Override
    public V get(Iterable<Character> keys) {
        Iterator<Character> it = keys.iterator();

        if (!it.hasNext()) {
            return null;
        }

        StringChild<V> child = get(it.next());
        if (child == null) {
            return null;
        }

        while (it.hasNext()) {
            StringChild<V> found = child.get(it.next());

            if (found == null) {
                break;
            }

            child = found;
        }

        return child.getValue();
    }

    @Override
    public V get(String keys) {
        return get(new IterableStringAdapter(keys));
    }

    @Override
    public StringChild<V> put(Character key, V value) {
        StringChild<V> child = getChildren().get(key);

        if (child == null) {
            child = buildChild(key, value);
            getChildren().put(key, child);
        } else if (value != null) {
            child.setValue(value);
        }

        return child;
    }

    @Override
    public StringChild<V> put(Iterable<Character> keys, V value) {
        Iterator<Character> it = keys.iterator();

        if (!it.hasNext()) {
            return null;
        }

        StringChild<V> current = put(it.next(), null);

        while (it.hasNext()) {
            current = current.put(it.next(), null);
        }

        current.setValue(value);
        return current;
    }

    @Override
    public StringChild<V> put(String keys, V value) {
        return put(new IterableStringAdapter(keys), value);
    }

    @Override
    public V remove(Character key) {
        StringChild<V> child = getChildren().get(key);
        if (child == null) {
            return null;
        }

        V value = child.getValue();

        if (child.hasChildren()) {
            child.setValue(null);
        } else {
            getChildren().remove(key);
        }

        return value;
    }

    @Override
    public V remove(Iterable<Character> keys) {
        Iterator<Character> it = keys.iterator();
        if (!it.hasNext()) {
            return null;
        }

        StringChild<V> current = get(it.next());
        if (current == null) {
            return null;
        }

        while (it.hasNext()) {
            current = current.get(it.next());

            if (current == null) {
                return null;
            }
        }

        V value = current.getValue();
        current.getParent().remove(current.getKey());

        return value;
    }

    @Override
    public V remove(String keys) {
        return remove(new IterableStringAdapter(keys));
    }
}
