package org.photon.shared.collect;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;

import java.util.List;
import java.util.Map;

/**
 * @author Blackrush
 */
public final class HashStringTree<V> extends AbstractStringTree<V> {
    public static <V> HashStringTree<V> create() {
        return new HashStringTree<>();
    }

    private final Map<Character, StringChild<V>> children = Maps.newHashMap();

    private HashStringTree() {}

    @Override
    protected Map<Character, StringChild<V>> getChildren() {
        return children;
    }

    @Override
    protected StringChild<V> buildChild(Character key, V value) {
        return new Child(this, key, value);
    }

    @Override
    public Tree<Character, V> getParent() {
        return null;
    }

    @Override
    public Tree<Character, V> getRoot() {
        return this;
    }

    public static class Child<V> extends AbstractStringTree<V> implements StringChild<V> {
        private final Map<Character, StringChild<V>> children = Maps.newHashMap();

        private final Tree<Character, V> parent;
        private final Character key;
        private V value;

        public Child(Tree<Character, V> parent, Character key, V value) {
            this.parent = parent;
            this.key = key;
            this.value = value;
        }

        @Override
        protected Map<Character, StringChild<V>> getChildren() {
            return children;
        }

        @Override
        protected StringChild<V> buildChild(Character key, V value) {
            return new HashStringTree.Child<>(this, key, value);
        }

        @Override
        public Character getKey() {
            return key;
        }

        @Override
        public Iterable<Character> getPath() {
            List<Character> builder = Lists.newArrayList(key);
            Tree<Character, V> parent = getParent();
            while (parent != null && parent instanceof Child<?, ?>) {
                builder.add(0, ((Child<Character, V>) parent).getKey());
                parent = parent.getParent();
            }
            return builder;
        }

        @Override
        public String getStringPath() {
            StringBuilder builder = new StringBuilder(key);
            Tree<Character, V> parent = getParent();
            while (parent != null && parent instanceof Child<?, ?>) {
                builder.insert(0, ((Child) parent).getKey());
                parent = parent.getParent();
            }
            return builder.toString();
        }

        @Override
        public V getValue() {
            return value;
        }

        @Override
        public void setValue(V value) {
            this.value = value;
        }

        @Override
        public Tree<Character, V> getParent() {
            return parent;
        }

        @Override
        public Tree<Character, V> getRoot() {
            return parent.getRoot();
        }
    }
}
