package org.photon.shared.inject;

import com.google.common.collect.Lists;
import com.google.inject.AbstractModule;
import com.google.inject.MembersInjector;
import com.google.inject.TypeLiteral;
import com.google.inject.matcher.Matchers;
import com.google.inject.spi.TypeEncounter;
import com.google.inject.spi.TypeListener;
import com.typesafe.config.Config;

import java.lang.reflect.Field;
import java.util.List;

/**
 * @author Blackrush
 */
public final class ConfigModule extends AbstractModule {
    private final Config config;

    private ConfigModule(Config config) {
        this.config = config;
    }

    public static ConfigModule of(Config config) {
        return new ConfigModule(config);
    }

    @Override
    protected void configure() {
        bind(Config.class).toInstance(config);

        bindListener(Matchers.any(), new Listener());
    }

    private void inject(Object target, List<Field> fields) {
        for (Field field : fields) {
            String key = field.getAnnotation(InjectConfig.class).value();
            try {
                field.set(target, config.getAnyRef(key));
            } catch (IllegalAccessException e) {
                addError(e);
            }
        }
    }

    private class Listener implements TypeListener {
        @Override
        public <I> void hear(TypeLiteral<I> type, TypeEncounter<I> encounter) {
            final List<Field> fields = Lists.newArrayList();
            for (Field field : type.getRawType().getDeclaredFields()) {
                if (field.isAnnotationPresent(InjectConfig.class)) {
                    fields.add(field);
                    field.setAccessible(true);
                }
            }

            if (fields.size() > 0) {
                encounter.register(new MembersInjector<I>() {
                    public void injectMembers(I instance) {
                        inject(instance, fields);
                    }
                });
            }
        }
    }
}
