package org.photon.shared.kernel;

import com.google.common.collect.Sets;
import com.google.inject.Module;
import com.google.inject.Stage;
import com.typesafe.config.Config;
import org.photon.shared.event.EventDispatcherStrategy;

import java.util.Collections;
import java.util.Set;

import static com.google.common.base.Preconditions.checkNotNull;

/**
 * @author Blackrush
 */
public final class BundleRegistries {
    private BundleRegistries() {}

    public static class Factory {
        private Stage stage;
        private Config config;
        private BundleLoader loader;
        private Set<Module> modules = Sets.newHashSet();
        private EventDispatcherStrategy dispatcherStrategy;

        public Factory withStage(Stage stage) {
            this.stage = stage;
            return this;
        }
        
        public Factory withConfig(Config config) {
            this.config = config;
            return this;
        }

        public Factory withLoader(BundleLoader loader) {
            this.loader = loader;
            return this;
        }

        public Factory withModule(Module module) {
            this.modules.add(module);
            return this;
        }

        public Factory withModules(Module module1, Module module2, Module... modules) {
            withModule(module1).withModule(module2);
            Collections.addAll(this.modules, modules);
            return this;
        }

        public Factory withDispatcherStrategy(EventDispatcherStrategy dispatcherStrategy) {
            this.dispatcherStrategy = dispatcherStrategy;
            return this;
        }

        public SimpleBundleRegistry simple() {
            return new SimpleBundleRegistry(stage, checkNotNull(config), checkNotNull(loader), modules, checkNotNull(dispatcherStrategy));
        }
    }

    public static Factory factory() {
        return new Factory();
    }

    public static Factory bundleRegistry() {
        return factory();
    }
}
