package org.photon.shared.kernel;

import com.google.inject.AbstractModule;
import com.google.inject.Injector;
import com.typesafe.config.Config;
import org.photon.shared.event.EventDispatcher;

/**
 * @author Blackrush
 */
public abstract class BundleRegistry implements Iterable<Bundle>, ServiceInterface {
    /**
     * try to find a bundle by its identifier
     * @param name bundle's identifier
     * @return found bundle by its identifier or null if not found
     */
    public abstract Bundle get(String name);

    /**
     * @return application wide config parameters
     */
    public abstract Config getConfig();

    /**
     * @return used injector to instantiate services
     */
    public abstract Injector getInjector();

    /**
     * @return application wide event dispatcher
     */
    public abstract EventDispatcher getDispatcher();

    /**
     * must be invoked before starting the {@link BundleRegistry}
     */
    public abstract void load();

    public <T extends ServiceInterface> T getService(Class<T> serviceClass) {
        return getInjector().getInstance(serviceClass);
    }

    protected final class BindThisRegistryModule extends AbstractModule {
        protected void configure() {
            bind(BundleRegistry.class).toInstance(BundleRegistry.this);
            bind(EventDispatcher.class).toInstance(getDispatcher());
        }
    }
}
