package org.photon.shared.event;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.JUnit4;

import java.util.Stack;

import static org.junit.Assert.assertTrue;

/**
 * @author Blackrush
 */
@RunWith(JUnit4.class)
public class ReflectEventDispatcherStrategyTest {
    public static interface Event {}
    public static class Event1 implements Event {}
    public static class Event2 implements Event {}
    public static class Event3 {}

    public static class Handler {
        Stack<Object> handled = new Stack<>();

        @Subscribe({Event1.class, Event2.class})
        public void handle(Event event) {
            handled.push(event);
        }

        @Subscribe
        public void handle1(Event1 event) {
            handled.push(event);
        }

        @Subscribe
        public void handle2(Event2 event) {
            handled.push(event);
        }

        @Subscribe
        public void handle3(Event3 event) {
            handled.push(event);
        }
    }
    public static class WrongHandler {
        @Subscribe
        public void handle(Event event, String wrongParameter) {}

        @Subscribe
        public void handle2(Event3 event) {}

        @Subscribe
        public void handle3(Event event) {}
    }

    private ReflectEventDispatcherStrategy strategy;

    @Before
    public void setUp() {
        strategy = EventDispatcherStrategies.reflect();
    }

    @Test
    public void testSuccessfulOnSubscribed() {
        strategy.onSubscribed(new Handler());
    }

    @Test(expected = ReflectEventDispatcherStrategy.Exception.class)
    public void testFailedOnSubscribed() {
        strategy.onSubscribed(new WrongHandler());
    }

    @Test
    public void testDispatch() {
        Handler h = new Handler();
        strategy.onSubscribed(h);

        strategy.doDispatch(new Event1(), h);

        assertTrue(h.handled.pop() instanceof Event1);
        assertTrue(h.handled.pop() instanceof Event1);
        assertTrue(h.handled.empty());

        strategy.doDispatch(new Event2(), h);

        assertTrue(h.handled.pop() instanceof Event2);
        assertTrue(h.handled.pop() instanceof Event2);
        assertTrue(h.handled.empty());

        strategy.doDispatch(new Event3(), h);

        assertTrue(h.handled.pop() instanceof Event3);
        assertTrue(h.handled.empty());
    }
}
